package dev.fyloz.colorrecipesexplorer.dtos

import com.fasterxml.jackson.annotation.JsonIgnore
import com.fasterxml.jackson.annotation.JsonProperty
import dev.fyloz.colorrecipesexplorer.Constants
import javax.validation.constraints.Min
import javax.validation.constraints.NotBlank

data class MixDto(
    override val id: Long = 0L,

    val location: String? = null,

    @JsonIgnore
    val recipeId: Long,

    val mixType: MixTypeDto,

    @JsonIgnore
    val mixQuantities: MixQuantitiesDto,
) : EntityDto {
    @Suppress("unused")
    @get:JsonProperty("mixQuantities")
    val mixQuantitiesOutput by lazy {
        mixQuantities.materials.map {
            MixQuantityOutputDto(it.id, it.material, it.quantity, it.position, false)
        } + mixQuantities.mixTypes.map {
            MixQuantityOutputDto(it.id, it.mixType.asMaterial(), it.quantity, it.position, true)
        }
    }
}

data class MixQuantitiesDto(
    val materials: List<MixMaterialDto> = listOf(),

    val mixTypes: List<MixMixTypeDto> = listOf()
) {
    val all get() = materials + mixTypes
}

data class MixQuantityOutputDto(
    val id: Long,

    val material: MaterialDto,

    val quantity: Float,

    val position: Int,

    val isMixType: Boolean
)

data class MixSaveDto(
    val id: Long = 0L,

    @field:NotBlank
    val name: String,

    val recipeId: Long = 0L,

    val materialTypeId: Long,

    val mixQuantities: List<MixQuantitySaveDto>
)

data class MixDeductDto(
    val id: Long,

    @field:Min(0, message = Constants.ValidationMessages.SIZE_GREATER_OR_EQUALS_ZERO)
    val ratio: Float
)

data class MixLocationDto(
    val mixId: Long,

    val location: String?
)