package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.account.Group
import dev.fyloz.colorrecipesexplorer.model.account.User
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository

@Repository
interface UserRepository : JpaRepository<User, Long> {
    /** Checks if a user with the given [firstName], [lastName] and a different [id] exists. */
    fun existsByFirstNameAndLastNameAndIdNot(firstName: String, lastName: String, id: Long): Boolean

    /** Finds all users for the given [group]. */
    @Query("SELECT u FROM User u WHERE u.group = :group AND u.isSystemUser IS FALSE AND u.isDefaultGroupUser IS FALSE")
    fun findAllByGroup(group: Group): Collection<User>

    /** Finds the user with the given [firstName] and [lastName]. */
    fun findByFirstNameAndLastName(firstName: String, lastName: String): User?

    /** Finds the default user for the given [group]. */
    @Query("SELECT u FROM User u WHERE u.group = :group AND u.isDefaultGroupUser IS TRUE")
    fun findDefaultGroupUser(group: Group): User?
}

@Repository
interface GroupRepository : JpaRepository<Group, Long> {
    /** Checks if a group with the given [name] and a different [id] exists. */
    fun existsByNameAndIdNot(name: String, id: Long): Boolean
}
