package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.Recipe
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.Query

interface RecipeRepository : JpaRepository<Recipe, Long> {
    /** Checks if a recipe with the given [name], [companyId] and a different [id] exists. */
    @Query(
        """
        SELECT CASE WHEN(COUNT(r) > 0) THEN TRUE ELSE FALSE END 
        FROM Recipe r WHERE r.name = :name AND r.company.id = :companyId AND r.id <> :id
        """
    )
    fun existsByNameAndCompanyAndIdNot(name: String, companyId: Long, id: Long): Boolean

    /** Gets all recipes with the given [name]. */
    fun findAllByName(name: String): Collection<Recipe>
}
