package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.config.annotations.ServiceComponent
import dev.fyloz.colorrecipesexplorer.dtos.GroupDto
import dev.fyloz.colorrecipesexplorer.model.account.Group
import dev.fyloz.colorrecipesexplorer.model.account.Permission
import dev.fyloz.colorrecipesexplorer.model.account.flat
import dev.fyloz.colorrecipesexplorer.repository.GroupRepository

interface GroupService : Service<GroupDto, Group, GroupRepository> {
    /** Checks if a group with the given [name] and a different [id] exists. */
    fun existsByName(name: String, id: Long? = null): Boolean

    /** Flatten the given the permissions of the given [group]. */
    fun flattenPermissions(group: Group): List<Permission>
}

@ServiceComponent
class DefaultGroupService(repository: GroupRepository) : BaseService<GroupDto, Group, GroupRepository>(repository),
    GroupService {
    override fun existsByName(name: String, id: Long?) = repository.existsByNameAndIdNot(name, id ?: 0L)

    override fun toDto(entity: Group) =
        GroupDto(entity.id, entity.name, flattenPermissions(entity), entity.permissions)

    override fun toEntity(dto: GroupDto) =
        Group(dto.id, dto.name, dto.permissions)

    override fun flattenPermissions(group: Group) =
        group.permissions.flatMap { it.flat() }.filter { !it.deprecated }
}