package dev.fyloz.colorrecipesexplorer.utils.collections

class LazyMapList<T, R>(private val sourceList: List<T>, private val transform: (T) -> R) : List<R> {
    private val list by lazy { sourceList.map(transform) }

    fun initialize() {
        // Call a property so the list is initialized
        size
    }

    override val size: Int
        get() = list.size

    override fun contains(element: R) = list.contains(element)
    override fun containsAll(elements: Collection<R>) = list.containsAll(elements)
    override fun get(index: Int) = list[index]
    override fun indexOf(element: R) = list.indexOf(element)
    override fun isEmpty() = list.isEmpty()
    override fun iterator() = list.iterator()
    override fun lastIndexOf(element: R) = list.lastIndexOf(element)
    override fun listIterator() = list.listIterator()
    override fun listIterator(index: Int) = list.listIterator(index)
    override fun subList(fromIndex: Int, toIndex: Int) = list.subList(fromIndex, toIndex)
}

fun <T, R> List<T>.lazyMap(transform: (T) -> R) = LazyMapList(this, transform)