package dev.fyloz.colorrecipesexplorer.utils

import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionsException
import io.mockk.clearAllMocks
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertDoesNotThrow
import org.junit.jupiter.api.assertThrows
import kotlin.test.assertTrue

class PositionUtilsTest {
    @AfterEach
    internal fun afterEach() {
        clearAllMocks()
    }

    @Test
    fun validateSteps_normalBehavior_doesNothing() {
        // Arrange
        val positions = listOf(1, 2)

        // Act
        // Assert
        assertDoesNotThrow { PositionUtils.validate(positions) }
    }

    @Test
    fun validateSteps_emptyStepSet_doesNothing() {
        // Arrange
        val positions = listOf<Int>()

        // Act
        // Assert
        assertDoesNotThrow { PositionUtils.validate(positions) }
    }

    @Test
    fun validateSteps_hasInvalidPositions_throwsInvalidStepsPositionsException() {
        // Arrange
        val positions = listOf(2, 3)

        // Act
        // Assert
        assertThrows<InvalidPositionsException> { PositionUtils.validate(positions) }
    }

    @Test
    fun validateSteps_firstStepPositionInvalid_returnsInvalidStepValidationError() {
        // Arrange
        val positions = listOf(2, 3)

        // Act
        val exception = assertThrows<InvalidPositionsException> { PositionUtils.validate(positions) }

        // Assert
        assertTrue {
            exception.errors.any { it.type == PositionUtils.INVALID_FIRST_POSITION_ERROR_CODE }
        }
    }

    @Test
    fun validateSteps_duplicatedPositions_returnsInvalidStepValidationError() {
        // Arrange
        val positions = listOf(1, 1)

        // Act
        val exception = assertThrows<InvalidPositionsException> { PositionUtils.validate(positions) }

        // Assert
        assertTrue {
            exception.errors.any { it.type == PositionUtils.DUPLICATED_POSITION_ERROR_CODE }
        }
    }

    @Test
    fun validateSteps_gapsInPositions_returnsInvalidStepValidationError() {
        // Arrange
        val positions = listOf(1, 3)

        // Act
        val exception = assertThrows<InvalidPositionsException> { PositionUtils.validate(positions) }

        // Assert
        assertTrue {
            exception.errors.any { it.type == PositionUtils.GAP_BETWEEN_POSITIONS_ERROR_CODE }
        }
    }
}